"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.help = exports.commandNames = exports.cliOptionsTypes = exports.rcOptionsTypes = exports.shorthands = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const cli_utils_1 = require("@pnpm/cli-utils");
const directory_fetcher_1 = require("@pnpm/directory-fetcher");
const fs_indexed_pkg_importer_1 = require("@pnpm/fs.indexed-pkg-importer");
const plugin_commands_installation_1 = require("@pnpm/plugin-commands-installation");
const error_1 = __importDefault(require("@pnpm/error"));
const rimraf_1 = __importDefault(require("@zkochan/rimraf"));
const render_help_1 = __importDefault(require("render-help"));
const deployHook_1 = require("./deployHook");
exports.shorthands = plugin_commands_installation_1.install.shorthands;
function rcOptionsTypes() {
    return plugin_commands_installation_1.install.rcOptionsTypes();
}
exports.rcOptionsTypes = rcOptionsTypes;
function cliOptionsTypes() {
    return plugin_commands_installation_1.install.cliOptionsTypes();
}
exports.cliOptionsTypes = cliOptionsTypes;
exports.commandNames = ['deploy'];
function help() {
    return (0, render_help_1.default)({
        description: 'Experimental! Deploy a package from a workspace',
        url: (0, cli_utils_1.docsUrl)('deploy'),
        usages: ['pnpm --filter=<deployed project name> deploy <target directory>'],
        descriptionLists: [
            {
                title: 'Options',
                list: [
                    {
                        description: "Packages in `devDependencies` won't be installed",
                        name: '--prod',
                        shortAlias: '-P',
                    },
                    {
                        description: 'Only `devDependencies` are installed regardless of the `NODE_ENV`',
                        name: '--dev',
                        shortAlias: '-D',
                    },
                    {
                        description: '`optionalDependencies` are not installed',
                        name: '--no-optional',
                    },
                ],
            },
        ],
    });
}
exports.help = help;
async function handler(opts, params) {
    if (!opts.workspaceDir) {
        throw new error_1.default('CANNOT_DEPLOY', 'A deploy is only possible from inside a workspace');
    }
    const selectedDirs = Object.keys(opts.selectedProjectsGraph ?? {});
    if (selectedDirs.length === 0) {
        throw new error_1.default('NOTHING_TO_DEPLOY', 'No project was selected for deployment');
    }
    if (selectedDirs.length > 1) {
        throw new error_1.default('CANNOT_DEPLOY_MANY', 'Cannot deploy more than 1 project');
    }
    if (params.length !== 1) {
        throw new error_1.default('INVALID_DEPLOY_TARGET', 'This command requires one parameter');
    }
    const deployedDir = selectedDirs[0];
    const deployDirParam = params[0];
    const deployDir = path_1.default.isAbsolute(deployDirParam) ? deployDirParam : path_1.default.join(opts.dir, deployDirParam);
    await (0, rimraf_1.default)(deployDir);
    await fs_1.default.promises.mkdir(deployDir, { recursive: true });
    await copyProject(deployedDir, deployDir);
    const readPackageHook = opts.hooks?.readPackage;
    // eslint-disable-next-line
    const newReadPackageHook = readPackageHook ? (async (pkg, context) => (0, deployHook_1.deployHook)(await readPackageHook(pkg, context))) : deployHook_1.deployHook;
    await plugin_commands_installation_1.install.handler({
        ...opts,
        depth: Infinity,
        hooks: {
            ...opts.hooks,
            readPackage: newReadPackageHook,
        },
        frozenLockfile: false,
        preferFrozenLockfile: false,
        saveLockfile: false,
        virtualStoreDir: path_1.default.join(deployDir, 'node_modules/.pnpm'),
        modulesDir: path_1.default.relative(deployedDir, path_1.default.join(deployDir, 'node_modules')),
        rawLocalConfig: {
            ...opts.rawLocalConfig,
            // This is a workaround to prevent frozen install in CI envs.
            'frozen-lockfile': false,
        },
    });
}
exports.handler = handler;
async function copyProject(src, dest) {
    const { filesIndex } = await (0, directory_fetcher_1.fetchFromDir)(src, { includeOnlyPackageFiles: true });
    const importPkg = (0, fs_indexed_pkg_importer_1.createIndexedPkgImporter)('clone-or-copy');
    await importPkg(dest, { filesMap: filesIndex, force: true, fromStore: true });
}
//# sourceMappingURL=deploy.js.map